@@ -2,4 +2,4 @@ from .base import BaseCFObject
 
 
 class Condition(BaseCFObject):
-    top_level_key = 'Conditions'
+    top_level_key = "Conditions"
