@@ -2,4 +2,4 @@ from .base import BaseCFObject
 
 
 class Metadata(BaseCFObject):
-    top_level_key = 'Metadata'
+    top_level_key = "Metadata"
