@@ -2,4 +2,4 @@ from .base import BaseCFObject
 
 
 class Transformation(BaseCFObject):
-    top_level_key = 'Transform'
+    top_level_key = "Transform"
