@@ -3,4 +3,4 @@ from .files import ComponentFile
 
 
 class Workflow(StepFunction):
-    test_payload = ComponentFile(name='test-payload.json', optional=True)
+    test_payload = ComponentFile(name="test-payload.json", optional=True)
