@@ -2,6 +2,9 @@ DEFAULT_CONFIG_FILENAME = "cirrus.yml"
 DEFAULT_SERVERLESS_FILENAME = "serverless.yml"
 DEFAULT_BUILD_DIR_NAME = ".cirrus"
 
+DEFAULT_DOT_DIR_NAME = ".cirrus"
+DEFAULT_BUILD_DIR_NAME = "build"
+
 DEFAULT_GIT_IGNORE = """__pycache__/
 node_modules/
 .DS_Store
