@@ -1,13 +1,12 @@
 import logging
 import os
 from datetime import datetime, timedelta, timezone
-from typing import Dict, List, Optional
+from typing import Any, Dict, List, Optional
 
 import boto3
 from boto3.dynamodb.conditions import Attr, Key
 
-# envvars
-PAYLOAD_BUCKET = os.getenv("CIRRUS_PAYLOAD_BUCKET")
+from .eventdb import EventDB, StateEnum
 
 STATES = ["PROCESSING", "COMPLETED", "FAILED", "INVALID", "ABORTED"]
 
@@ -15,25 +14,46 @@ STATES = ["PROCESSING", "COMPLETED", "FAILED", "INVALID", "ABORTED"]
 logger = logging.getLogger(__name__)
 
 
+def get_payload_bucket():
+    return os.getenv("CIRRUS_PAYLOAD_BUCKET")
+
+
+def get_state_db():
+    return os.getenv("CIRRUS_STATE_DB")
+
+
 class StateDB:
     limit = None
 
-    def __init__(self, table_name: str = None):
+    def __init__(
+        self,
+        table_name: Optional[str] = None,
+        eventdb: Optional[EventDB] = None,
+        session: Optional[boto3.Session] = None,
+    ):
         """Initialize a StateDB instance using the Cirrus State DB table
 
         Args:
             table_name (str, optional): The Cirrus StateDB Table name. Defaults to os.getenv('CIRRUS_STATE_DB', None).
         """
-        table_name = table_name if table_name else os.getenv("CIRRUS_STATE_DB")
+        table_name = table_name if table_name else get_state_db()
 
         if not table_name:
             raise ValueError("env var CIRRUS_STATE_DB must be defined")
 
+        if not session:
+            session = boto3.Session()
+
         # initialize client
-        self.db = boto3.resource("dynamodb")
+        self.db = session.resource("dynamodb")
         self.table_name = table_name
         self.table = self.db.Table(table_name)
 
+        if eventdb:
+            self.eventdb = eventdb
+        else:
+            self.eventdb = EventDB(session=session)
+
     def delete_item(self, payload_id: str):
         key = self.payload_id_to_key(payload_id)
         response = self.table.delete_item(Key=key)
@@ -82,7 +102,7 @@ class StateDB:
             resp = self.db.meta.client.batch_get_item(
                 RequestItems={
                     self.table_name: {
-                        "Keys": [self.payload_id_to_key(id) for id in set(payload_ids)]
+                        "Keys": [self.payload_id_to_key(x) for x in set(payload_ids)]
                     }
                 }
             )
@@ -115,9 +135,8 @@ class StateDB:
         query_kwargs["collections_workflow"] = collections_workflow
         query_kwargs["select"] = "COUNT"
 
-        counts = 0
         resp = self.query(**query_kwargs)
-        counts = resp["Count"]
+        counts = resp.get("Count", 0)
 
         while "LastEvaluatedKey" in resp and (not limit or counts <= limit):
             query_kwargs["ExclusiveStartKey"] = resp["LastEvaluatedKey"]
@@ -135,7 +154,7 @@ class StateDB:
         limit=100,
         nextkey: str = None,
         **kwargs,
-    ) -> List[Dict]:
+    ) -> Dict[str, Any]:
         """Get Items by query
 
         Args:
@@ -148,7 +167,7 @@ class StateDB:
         Returns:
             Dict: List of Items
         """
-        items = {"items": []}
+        items: Dict[str, Any] = {"items": []}
         kwargs.update(
             {
                 "collections_workflow": collections_workflow,
@@ -199,10 +218,8 @@ class StateDB:
 
     def get_state(self, payload_id: str) -> str:
         """Get current state of Item
-
         Args:
             payload_id (str): The Payload ID
-
         Returns:
             str: Current state: PROCESSING, COMPLETED, FAILED, INVALID, ABORTED
         """
@@ -215,10 +232,8 @@ class StateDB:
 
     def get_states(self, payload_ids: List[str]) -> Dict[str, str]:
         """Get current state of items
-
         Args:
             payload_ids (List[str]): List of Payload IDs
-
         Returns:
             Dict[str, str]: Dictionary of Payload IDs to state
         """
@@ -252,7 +267,7 @@ class StateDB:
         logger.debug("Claimed processing", extra=key)
         return response
 
-    def set_processing(self, payload_id, execution):
+    def set_processing(self, payload_id: str, execution_arn: str) -> Dict[str, Any]:
         """Adds execution to existing item or creates new"""
         now = datetime.now(timezone.utc).isoformat()
         key = self.payload_id_to_key(payload_id)
@@ -271,14 +286,17 @@ class StateDB:
                 ":state_updated": f"PROCESSING_{now}",
                 ":updated": now,
                 ":empty_list": [],
-                ":exes": [execution],
+                ":exes": [execution_arn],
             },
         )
-        logger.debug("Add execution", extra=key.update({"execution": execution}))
+        logger.debug("Add execution", extra=key.update({"execution": execution_arn}))
+
+        self.write_timeseries_record(key, StateEnum.PROCESSING, now, execution_arn)
+
         return response
 
     def set_outputs(self, payload_id: str, outputs: List[str]) -> str:
-        """Set this item as COMPLETED
+        """Set this item's outputs
 
         Args:
             payload_id (str): The Cirrus Payload
@@ -309,13 +327,17 @@ class StateDB:
         return response
 
     def set_completed(
-        self, payload_id: str, outputs: Optional[List[str]] = None
+        self,
+        payload_id: str,
+        outputs: Optional[List[str]] = None,
+        execution_arn: Optional[str] = None,
     ) -> str:
         """Set this item as COMPLETED
 
         Args:
             payload_id (str): The Cirrus Payload
             outputs (Optional[[str]], optional): List of URLs to output Items. Defaults to None.
+            execution_arn (Optional[str]): The Step Function execution ARN.
 
         Returns:
             str: DynamoDB response
@@ -343,10 +365,16 @@ class StateDB:
             UpdateExpression=expr,
             ExpressionAttributeValues=expr_attrs,
         )
-        logger.debug("set outputs", extra=key.update({"outputs": outputs}))
+        logger.debug("set completed", extra=key.update({"outputs": outputs}))
+
+        if execution_arn:
+            self.write_timeseries_record(key, StateEnum.COMPLETED, now, execution_arn)
+        else:
+            logger.debug("set completed called with no execution ARN")
+
         return response
 
-    def set_failed(self, payload_id, msg):
+    def set_failed(self, payload_id, msg, execution_arn: Optional[str] = None):
         """Adds new item as failed"""
         """ Adds new item with state function execution """
         now = datetime.now(timezone.utc).isoformat()
@@ -369,14 +397,21 @@ class StateDB:
             },
         )
         logger.debug("set failed", extra=key.update({"last_error": msg}))
+
+        if execution_arn:
+            self.write_timeseries_record(key, StateEnum.FAILED, now, execution_arn)
+
         return response
 
-    def set_invalid(self, payload_id: str, msg: str) -> str:
+    def set_invalid(
+        self, payload_id: str, msg: str, execution_arn: Optional[str] = None
+    ) -> str:
         """Set this item as INVALID
 
         Args:
             payload_id (str): The Cirrus Payload
             msg (str): An error message to include in DynamoDB Item
+            execution_arn (Optional[str]): The Step Function execution ARN.
 
         Returns:
             str: DynamoDB response
@@ -401,13 +436,18 @@ class StateDB:
             },
         )
         logger.debug("set invalid", extra=key.update({"last_error": msg}))
+
+        if execution_arn:
+            self.write_timeseries_record(key, StateEnum.INVALID, now, execution_arn)
+
         return response
 
-    def set_aborted(self, payload_id: str) -> str:
+    def set_aborted(self, payload_id: str, execution_arn: Optional[str] = None) -> str:
         """Set this item as ABORTED
 
         Args:
             payload_id (str): The Cirrus Payload
+            execution_arn (Optional[str]): The Step Function execution ARN.
 
         Returns:
             str: DynamoDB response
@@ -429,7 +469,12 @@ class StateDB:
                 ":updated": now,
             },
         )
+
         logger.debug("set aborted")
+
+        if execution_arn:
+            self.write_timeseries_record(key, StateEnum.ABORTED, now, execution_arn)
+
         return response
 
     def query(
@@ -518,7 +563,32 @@ class StateDB:
         return resp
 
     @classmethod
-    def payload_id_to_key(cls, payload_id: str) -> Dict:
+    def dbitem_to_item(
+        cls, dbitem: Dict, region: str = os.getenv("AWS_REGION", "us-west-2")
+    ) -> Dict:
+        state, updated = dbitem["state_updated"].split("_")
+        collections, workflow = dbitem["collections_workflow"].rsplit("_", maxsplit=1)
+        item = {
+            "payload_id": cls.key_to_payload_id(dbitem),
+            "collections": collections,
+            "workflow": workflow,
+            "items": dbitem["itemids"],
+            "state": state,
+            "created": dbitem["created"],
+            "updated": dbitem["updated"],
+            "payload": cls.payload_id_to_url(cls.key_to_payload_id(dbitem)),
+        }
+        if "executions" in dbitem:
+            base_url = f"https://{region}.console.aws.amazon.com/states/home?region={region}#/v2/executions/details/"
+            item["executions"] = [base_url + f"{e}" for e in dbitem["executions"]]
+        if "outputs" in dbitem:
+            item["outputs"] = dbitem["outputs"]
+        if "last_error" in dbitem:
+            item["last_error"] = dbitem["last_error"]
+        return item
+
+    @staticmethod
+    def payload_id_to_key(payload_id: str) -> Dict:
         """Create DynamoDB Key from Payload ID
 
         Args:
@@ -535,8 +605,8 @@ class StateDB:
         }
         return key
 
-    @classmethod
-    def key_to_payload_id(cls, key: Dict) -> str:
+    @staticmethod
+    def key_to_payload_id(key: Dict) -> str:
         """Get Payload ID given a DynamoDB Key
 
         Args:
@@ -548,48 +618,29 @@ class StateDB:
         parts = key["collections_workflow"].rsplit("_", maxsplit=1)
         return f"{parts[0]}/workflow-{parts[1]}/{key['itemids']}"
 
+    @staticmethod
+    def payload_id_to_bucket_key(payload_id, payload_bucket=None):
+        if not payload_bucket:
+            payload_bucket = get_payload_bucket()
+        return (payload_bucket, f"{payload_id}/input.json")
+
     @classmethod
-    def get_input_payload_url(cls, dbitem):
-        payload_id = cls.key_to_payload_id(dbitem)
-        return cls.get_payload_url(payload_id)
+    def payload_id_to_url(cls, payload_id, payload_bucket=None):
+        bucket, key = cls.payload_id_to_bucket_key(
+            payload_id,
+            payload_bucket=payload_bucket,
+        )
+        return f"s3://{bucket}/{key}"
+
+    @classmethod
+    def payload_key_to_url(cls, key, payload_bucket=None):
+        return cls.payload_id_to_url(
+            cls.key_to_payload_id(key),
+            payload_bucket=payload_bucket,
+        )
 
     @staticmethod
-    def get_payload_url(payload_id):
-        # TODO: util fn to get env vars or throw exception
-        payload_bucket = os.getenv("CIRRUS_PAYLOAD_BUCKET")
-
-        if not payload_bucket:
-            raise ValueError("env var CIRRUS_PAYLOAD_BUCKET must be defined")
-
-        return f"s3://{PAYLOAD_BUCKET}/{payload_id}/input.json"
-
-    @classmethod
-    def dbitem_to_item(
-        cls, dbitem: Dict, region: str = os.getenv("AWS_REGION", "us-west-2")
-    ) -> Dict:
-        state, updated = dbitem["state_updated"].split("_")
-        collections, workflow = dbitem["collections_workflow"].rsplit("_", maxsplit=1)
-        item = {
-            "payload_id": cls.key_to_payload_id(dbitem),
-            "collections": collections,
-            "workflow": workflow,
-            "items": dbitem["itemids"],
-            "state": state,
-            "created": dbitem["created"],
-            "updated": dbitem["updated"],
-            "payload": cls.get_input_payload_url(dbitem),
-        }
-        if "executions" in dbitem:
-            base_url = f"https://{region}.console.aws.amazon.com/states/home?region={region}#/executions/details/"
-            item["executions"] = [base_url + f"{e}" for e in dbitem["executions"]]
-        if "outputs" in dbitem:
-            item["outputs"] = dbitem["outputs"]
-        if "last_error" in dbitem:
-            item["last_error"] = dbitem["last_error"]
-        return item
-
-    @classmethod
-    def since_to_timedelta(cls, since: str) -> timedelta:
+    def since_to_timedelta(since: str) -> timedelta:
         """Convert a `since` field to a timedelta.
 
         Args:
@@ -605,3 +656,13 @@ class StateDB:
         hours = int(since[0:-1]) if unit == "h" else 0
         minutes = int(since[0:-1]) if unit == "m" else 0
         return timedelta(days=days, hours=hours, minutes=minutes)
+
+    def write_timeseries_record(
+        self,
+        key: Dict[str, str],
+        state: StateEnum,
+        event_time: str,
+        execution_arn: str,
+    ) -> None:
+        if self.eventdb:
+            self.eventdb.write_timeseries_record(key, state, event_time, execution_arn)
